package org.xbib.interlibrary.client.gbv;

import org.xbib.interlibrary.action.avail.AvailService;
import org.xbib.interlibrary.action.avail.AvailServiceProvider;
import org.xbib.interlibrary.action.resolve.ResolveService;
import org.xbib.interlibrary.action.resolve.ResolveServiceProvider;
import org.xbib.interlibrary.action.search.SearchService;
import org.xbib.interlibrary.action.search.SearchServiceProvider;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.client.gbv.sru.SRUAvailService;
import org.xbib.interlibrary.client.gbv.sru.SRUResolveService;
import org.xbib.interlibrary.client.gbv.sru.SRUSearchService;
import org.xbib.interlibrary.client.gbv.z.ZAvailService;
import org.xbib.interlibrary.client.gbv.z.ZResolveService;
import org.xbib.interlibrary.client.gbv.z.ZSearchService;
import org.xbib.interlibrary.sru.SRUClientHelper;

public class Provider {

    volatile static SRUClientHelper clientHelper;

    public Provider() {
    }

    public static class SRUResolveServiceProvider implements ResolveServiceProvider {

        private SRUResolveService sruResolveService;

        public SRUResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruResolveService == null) {
                sruResolveService = new SRUResolveService(arguments, clientHelper);
            }
            return sruResolveService;
        }
    }

    public static class SRUAvailServiceProvider implements AvailServiceProvider {

        private SRUAvailService sruAvailService;

        public SRUAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruAvailService == null) {
                sruAvailService = new SRUAvailService(arguments, clientHelper);
            }
            return sruAvailService;
        }
    }

    public static class SRUSearchServiceProvider implements SearchServiceProvider {

        private SRUSearchService sruSearchService;

        public SRUSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruSearchService == null) {
                sruSearchService = new SRUSearchService(arguments, clientHelper);
            }
            return sruSearchService;
        }
    }

    public static class ZAvailServiceProvider implements AvailServiceProvider {

        private ZAvailService zAvailService;

        public ZAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (zAvailService == null) {
                zAvailService = new ZAvailService(arguments);
            }
            return zAvailService;
        }
    }

    public static class ZResolveServiceProvider implements ResolveServiceProvider {

        private ZResolveService zResolveService;

        public ZResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (zResolveService == null) {
                zResolveService = new ZResolveService(arguments);
            }
            return zResolveService;
        }
    }

    public static class ZSearchServiceProvider implements SearchServiceProvider {

        private ZSearchService zSearchService;

        public ZSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (zSearchService == null) {
                zSearchService = new ZSearchService(arguments);
            }
            return zSearchService;
        }
    }
}
