package org.xbib.interlibrary.client.gbv.sru;

import java.util.concurrent.atomic.AtomicLong;
import java.util.function.Supplier;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.search.SearchRequest;
import org.xbib.interlibrary.action.search.SearchResponse;
import org.xbib.interlibrary.action.search.SearchResponseBuilder;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.sru.AbstractSRUSearchService;
import org.xbib.interlibrary.sru.DisplayHelper;
import org.xbib.interlibrary.sru.SRUClientHelper;
import org.xbib.interlibrary.sru.SRUContentHandler;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.MarcRecordListener;
import org.xbib.marc.MarcXchangeConstants;
import org.xbib.marc.xml.MarcContentHandler;
import org.xml.sax.SAXException;

public class SRUSearchService extends AbstractSRUSearchService {

    public SRUSearchService(ServiceArguments arguments, SRUClientHelper sruClientHelper) {
        super(arguments, sruClientHelper);
    }

    @Override
    protected Supplier<String> getIdSupplier(MarcRecord marcRecord) {
        return marcHelper.getRecordIdSupplier(marcRecord);
    }

    @Override
    protected SRUContentHandler getContentHandler(MarcContentHandler marcContentHandler) {
        return new GBVContentHandler(marcContentHandler);
    }

    @Override
    protected MarcRecordListener getMarcRecordListener(SearchRequest searchRequest,
                                                       SearchResponseBuilder searchResponse,
                                                       AtomicLong count) {
        return new MarcRecordListener() {

            @Override
            public void beginCollection() {
            }

            @Override
            public void record(MarcRecord marcRecord) {
                Supplier<String> id = getIdSupplier(marcRecord);
                TinyMap.Builder<String, Object> map = DisplayHelper.display(marcHelper, marcRecord, getName(), getLabel(), getURL(id.get()), id);
                map.put("_index", clientHelper.getIndex());
                map.put("_id", Long.toString(count.incrementAndGet()));
                map.put("_leader", marcRecord.getRecordLabel().toString());
                if (searchRequest.isSourceEnabled()) {
                    map.put("_source", marcRecord);
                }
                logSource(marcRecord);
                searchResponse.add(getName(), map.build());
            }

            @Override
            public void endCollection() {
            }
        };
    }

    @Override
    public int compareTo(Service<SearchRequest, SearchResponse<SearchRequest>, SearchResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }

    @Override
    protected boolean fixByOne() {
        return true;
    }

    private static class GBVContentHandler extends MarcContentHandler implements SRUContentHandler {

        private Long numberOfRecords;

        private GBVContentHandler(MarcContentHandler marcContentHandler) {
            super();
            super.setFormat("MarcXML")
                    .setType(MarcXchangeConstants.BIBLIOGRAPHIC_TYPE)
                    .addNamespace("http://www.loc.gov/MARC21/slim")
                    .setMarcListener(MarcXchangeConstants.BIBLIOGRAPHIC_TYPE, marcContentHandler);
        }

        @Override
        public void endElement(String uri, String localName, String qName) throws SAXException {
            if ("numberOfRecords".equals(localName)) {
                try {
                    this.numberOfRecords = Long.parseLong(content.toString());
                } catch (NumberFormatException e) {
                    // ignore
                }
            }
            super.endElement(uri, localName, qName);
        }

        @Override
        public Long getNumberOfRecords() {
            return numberOfRecords != null ? numberOfRecords : 0L;
        }
    }
}
