package org.xbib.interlibrary.client.gbv.z;

import org.xbib.interlibrary.action.resolve.ResolveRequest;
import org.xbib.interlibrary.action.resolve.ResolveResponse;
import org.xbib.interlibrary.action.resolve.ResolveResponseBuilder;
import org.xbib.interlibrary.action.resolve.ResolveServiceResponseBuilder;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.common.DefaultBibliographicDescription;
import org.xbib.interlibrary.z.AbstractZResolveService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.util.Locale;
import java.util.logging.Logger;

public class ZResolveService extends AbstractZResolveService {

    private static final Logger logger = Logger.getLogger(ZResolveService.class.getName());

    public ZResolveService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("gbv") ? "GBV" : null;
    }

    @Override
    protected void resolve(ResolveRequest resolveRequest,
                           ResolveServiceResponseBuilder serviceResponseBuilder,
                           DefaultBibliographicDescription.Builder descriptionBuilder,
                           MarcRecord marcRecord) {
        for (MarcField marcField : marcRecord.getAll("025")) {
            String zdbID = marcField.getFirstSubfieldValue("z");
            if (zdbID != null) {
                descriptionBuilder.setSource("ZDB");
                descriptionBuilder.setSourceId(zdbID);
            }
        }
    }

    @Override
    public int compareTo(Service<ResolveRequest, ResolveResponse, ResolveResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
